##!/usr/bin/python

#########################################################
# LIS3DH library
#
# Name: Mengcheng Qi, Wendian Jiang
# NetID: mq58, wj225
# Lab Section: Wednesday 4:30 - 7:30
#
##########################################################

import Adafruit_GPIO.I2C as I2C
import RPi.GPIO as GPIO        #needed for Hardware interrupt

class LIS3DH:

   # import LIS3DH parameters
   from LIS3DH_PARAM import *

   i2c = None
   
   def __init__(self, address=0x18, bus=1, debug=False):

      # i2c setup
      self.i2c = I2C.Device(address, busnum=bus, i2c_interface = None)
      self.address = address

      # Enable all axis (optional)
      self.setAxisStatus(True)

      # Set 400Hz refresh rate
      self.setDataRate(self.DATARATE_400HZ)
      
      # Set high resolution
      self.setHighResolution()

      # Enbale BDU (optional)
      self.setBDU()

      # Set 2g g-range
      self.setRange(self.RANGE_2G)
   
   ##############################
   #  general configuration     #
   ##############################

   # Set the range of the sensor (2G, 4G, 8G, 16G)
   def setRange(self, range):
      val = self.readRegister(self.REG_CTRL4) # Get value from register
      val &= ~(0b110000) # Mask off lowest 4 bits
      val |= (range << 4) # Write in our new range
      self.writeRegister(self.REG_CTRL4, val) # Write back to register
   
   # Set the rate (cycles per second) at which data is gathered
   def setDataRate(self, dataRate):
      val = self.readRegister(self.REG_CTRL1) # Get current value
      val &= 0b1111 # Mask off lowest 4 bits
      val |= (dataRate << 4) # Write in our new data rate to highest 4 bits
      self.writeRegister(self.REG_CTRL1, val) # Write back to register

   # Set whether we want to use high resolution or not
   def setHighResolution(self, highRes=True):
      val = self.readRegister(self.REG_CTRL4) # Get current value
      status = 0 if highRes else 1
      final = self.setBit(val, 3, status) # High resolution is bit 4 of REG_CTRL4
      self.writeRegister(self.REG_CTRL4, final)

   # Set whether we want to use block data update or not
   # False = output registers not updated until MSB and LSB reading
   def setBDU(self, bdu=True):
      val = self.readRegister(self.REG_CTRL4) # Get current value
      status = 1 if bdu else 0
      final = self.setBit(val, 7, status) # Block data update is bit 8 of REG_CTRL4
      self.writeRegister(self.REG_CTRL4, final)

   ##############################
   #  Raw Data                  #
   ##############################

   # Enable or disable an individual axis
   # Read status from CTRL_REG1, then write back with appropriate status bit changed
   def setAxisStatus(self, enable):		
       current = self.readRegister(self.REG_CTRL1)
       status = 1 if enable else 0
       final = self.setBit(current, self.AXIS_X, status)
       final = self.setBit(current, self.AXIS_Y, status)
       final = self.setBit(current, self.AXIS_Z, status)
       self.writeRegister(self.REG_CTRL1, final)
       
   # Get a reading from the desired axis
   def getAxis(self, axis):
      base = self.REG_OUT_X_L + (2 * axis) # Determine which register we need to read from (2 per axis)
      low = self.i2c.readU8(base) # Read the first register (lower bits)
      high = self.i2c.readU8(base + 1) # Read the next register (higher bits)
      res = low | (high << 8) # Combine the two components
      res = self.twosComp(res) # Calculate the twos compliment of the result
      return res

   # Get reading axis
   def getRawdata(self):
      return self.getAxis(self.AXIS_X),self.getAxis(self.AXIS_Y),self.getAxis(self.AXIS_Z)


   ##############################
   #  Single/Double Click       #
   ##############################
	   	
   def setClick(self,clickmode,clickthresh=0X7F,timelimit=10,timelatency=20,timewindow=200,mycallback=None):
         
        self.writeRegister(self.REG_CTRL3, 0x80)  # turn on int1 click
        self.writeRegister(self.REG_CTRL5, 0x08)  # latch interrupt on int1
		
        if (clickmode == self.CLK_SINGLE):
            self.writeRegister(self.REG_CLICKCFG, 0x15) # turn on all axes & singletap
        elif (clickmode == self.CLK_DOUBLE):
            self.writeRegister(self.REG_CLICKCFG, 0x2A) # turn on all axes & doubletap
        elif (clickmode == self.CLK_ALL):
            self.writeRegister(self.REG_CLICKCFG, 0x3F) # turn on all axes & tap

        # set timing and threshold parameters
        self.writeRegister(self.REG_CLICKTHS, clickthresh)
        self.writeRegister(self.REG_TIMELIMIT, timelimit)
        self.writeRegister(self.REG_TIMELATENCY, timelatency)
        self.writeRegister(self.REG_TIMEWINDOW, timewindow)

        if mycallback != None:
            self.setInterrupt(mycallback)

   def getClick(self):
        reg = self.readRegister(self.REG_CLICKSRC)       # read click register
        #self.readRegister(self.REG_INT1SRC)              # reset  interrupt flag
        return reg


   ##############################
   #  Orientation detect        #
   ##############################
   def setorient(self):
   	  # latch interrupt on int1
      self.writeRegister(self.REG_CTRL5, 0x0C)  

      # Set mode and threhold parameters
      self.writeRegister(self.REG_INT1THS, 0x20) 
      self.writeRegister(self.REG_INT1CFG, 0xCF)
      

   def getorient(self):
      return self.readRegister(self.REG_INT1SRC)   # read orientation register
      

   ##############################
   #  others                    #
   ##############################
   
   # Write the given value to the given register
   def writeRegister(self, register, value):
      self.i2c.write8(register, value)

   # Read from the given register
   def readRegister(self, register):
      return self.i2c.readU8(register)

   # Set the bit at index 'bit' to 'value' on 'input' and return
   def setBit(self, input, bit, value):
      mask = 1 << bit
      input &= ~mask
      if value:
         input |= mask
      return input

   # Return a 16-bit signed number (two's compliment)
   def twosComp(self,x) :
      if (0x8000 & x): 
         x = - (0x010000 - x)
      return x

   def setInterrupt(self,mycallback):
        GPIO.setmode(GPIO.BCM)
        GPIO.setup(self.INT_IO, GPIO.IN)
        GPIO.add_event_detect(self.INT_IO, GPIO.RISING, callback=mycallback)

